# -*- coding: utf8 -*-

#-----------------------------------------
# This is a part of Collège 3D project.
# Author:       Pascal Peter
# Copyright:    (C) 2003-2016 Pascal Peter
# Licence:      GNU General Public Licence version 3
# Website:      http://pascal.peter.free.fr/
# Email:        pascal.peter at free.fr
#-----------------------------------------

"""
DESCRIPTION :
    Affichage du menu.
"""

# **************************************************
#       IMPORTATION DES MODULES UTILES
# **************************************************
import sys

import bge

import utils, input_controller



# **************************************************
#       CONTRÔLES DU JEU
# **************************************************
controller = None
scene = None



# **************************************************
#       LES ITEMS DU MENU
# **************************************************
menuItems = {
    #MENU: (NOM, COORDONNÉES, COCHABLE, TAILLE, COMMENTAIRE, NOMDANSDICO)
    'item_1': (
        'GAME', (0.35, 0.12), False, 2.5, 'Select the game', 'gameName'),
    'item_2': (
        'CHARACTER', (0.05, 0.28), False, 1.5,
        'Select the character', 'character'),
    'item_3': (
        'MODE', (0.05, 0.36), False, 1.5,
        'Select the mode   (keyboard, mouse, ...)', 'mode'),
    'item_4': (
        'LANG_NAME', (0.05, 0.44), False, 1.5, 'Select the langage', 'lang'),

    'item_11': (
        'Sounds', (0.4, 0.28), True, 1.5, '', 'sounds'),
    'item_12': 
        ('Music', (0.4, 0.36), True, 1.5, '', 'music'),
    'item_13': (
        'Anaglyphs', (0.4, 0.44), True, 1.5,
        'You must reload after changing this option', 'anaglyphs'),
    #'item_14': (
    #    'Fullscreen', (0.4, 0.52), True, 1.5,
    #    'You must reload after changing this option', 'fullScreen'),
    'item_15': (
        'Infos', (0.4, 0.60), True, 1.5,
        '', 'infos'),
    #'item_16': (
    #    'Video', (0.4, 0.68), True, 1.5,
    #    'You must reload after changing this option', 'video'),

    'item_21': (
        'Help', (0.75, 0.28), False, 1.5, 'Game Help', ''),
    'item_22': (
        'About', (0.75, 0.36), False, 1.5, 'Project Web Page', ''),
    'item_23': (
        'Server', (0.75, 0.52), False, 1.5, 
        'Set this computer as LAN server', ''),
    'item_24': (
        'Characters', (0.75, 0.60), False, 1.5, 
        'Test characters', ''),
    #'item_25': (
    #    'Blender', (0.75, 0.68), False, 1.5, 
    #    'Launch Blender (game embedded version)', ''),

    'item_91': (
        'PLAY', (0.35, 0.95), False, 3, '', ''),
    'item_92': (
        'EXIT', (0.65, 0.95), False, 3, '', ''),
    }

menuItemsList = (
    'item_1', 'item_2', 'item_3', 'item_4', 
    'item_11', 'item_12', 'item_13', #'item_14', 
    'item_15', 
    'item_21', 'item_22', 'item_23', 'item_24', #'item_25', 
    'item_91', 'item_92')



# **************************************************
#       FONCTIONS UTILES
# **************************************************
def postDrawTexts():
    """
    affichage des textes.
    """
    utils.postDrawTexts('menu_loop')

def doAction(what):
    """
    lance l'action correspondant au menu qui a été cliqué.
    """
    menuItemName = 'item_{0}'.format(what)
    lang = bge.logic.globalDict['MENU']['lang'] 
    if menuItems[menuItemName][2]:
        # si le menu est cochable, on met son état à jour :
        menuKey = menuItems[menuItemName][5]
        bge.logic.globalDict['MENU'][menuKey] = not(bge.logic.globalDict['MENU'][menuKey])
        menuText = utils.tr('menu', menuItems[menuItemName][0], lang)
        if bge.logic.globalDict['MENU'][menuKey]:
            menuText = '[X]    {0}'.format(menuText)
        else:
            menuText = '[ ]    {0}'.format(menuText)
        utils.setText(menuItemName, text=menuText)
    elif what == 92:
        # quitter :
        bge.logic.globalDict['GAME']['state'] = 'menu_quit'
        utils.saveSettings()
        scene.replace('quit')
    elif what == 91:
        # lancer le jeu :
        bge.logic.globalDict['GAME']['state'] = 'menu_quit'
        utils.saveSettings()
        gameName = bge.logic.globalDict['MENU']['gameName']
        bge.logic.globalDict['GAME']['levels'] = {}
        bge.logic.globalDict['GAME']['createdOjects'] = []
        if bge.logic.globalDict['GAMES'][gameName] != []:
            theScene = 'levels'
        elif gameName in ('LAN', ):
            theScene = 'lan_game'
        else:
            theScene = 'game'
        scene.replace(theScene)
    elif what == 1:
        # choix du jeu :
        bge.logic.globalDict['MENU']['gameName'] = utils.nextInList(
            bge.logic.globalDict['GAMES']['LIST'],
            bge.logic.globalDict['MENU']['gameName'])
        menuText = utils.tr('games', bge.logic.globalDict['MENU']['gameName'], lang)
        utils.setText(menuItemName, text=menuText)
    elif what == 2:
        # choix du personnage :
        bge.logic.globalDict['MENU']['character'] = utils.nextInList(
            utils.characters, bge.logic.globalDict['MENU']['character'])
        menuText = utils.tr('menu', bge.logic.globalDict['MENU']['character'], lang)
        utils.setText(menuItemName, text=menuText)
    elif what == 3:
        # choix du mode :
        bge.logic.globalDict['MENU']['mode'] = utils.nextInList(
            utils.modes, bge.logic.globalDict['MENU']['mode'])
        menuText = utils.tr('menu', bge.logic.globalDict['MENU']['mode'], lang)
        utils.setText(menuItemName, text=menuText)
    elif what == 4:        
        # langue suivante :
        bge.logic.globalDict['MENU']['lang'] = utils.nextInList(
            bge.logic.globalDict['TRANSLATIONS']['languages'], 
            bge.logic.globalDict['MENU']['lang'])
        lang = bge.logic.globalDict['MENU']['lang']
        # on doit recharger les textes de tous les menus :
        for menuItemName2 in menuItems:
            what2 = int(menuItemName2[5:])
            if what2 == 1:
                menuText = utils.tr('games', bge.logic.globalDict['MENU']['gameName'], lang)
            elif what2 == 2:
                menuText = utils.tr('menu', bge.logic.globalDict['MENU']['character'], lang)
            elif what2 == 3:
                menuText = utils.tr('menu', bge.logic.globalDict['MENU']['mode'], lang)
            elif what2 == 4:
                menuText = utils.tr('menu', 'LANG_NAME', lang)
            else:
                menuText = utils.tr('menu', menuItems[menuItemName2][0], lang)
            # si le menu est cochable, on initialise son état :
            if menuItems[menuItemName2][2]:
                menuKey = menuItems[menuItemName2][5]
                if bge.logic.globalDict['MENU'][menuKey]:
                    menuText = '[X]    {0}'.format(menuText)
                else:
                    menuText = '[ ]    {0}'.format(menuText)
            utils.setText(menuItemName2, text=menuText)
    elif what == 21:
        # aide :
        import webbrowser
        webbrowser.open(utils.WEBPAGE_HELP)
    elif what == 22:
        # à propos :
        import webbrowser
        webbrowser.open(utils.WEBPAGE_ABOUT)
    elif what == 23:
        # serveur réseau :
        bge.logic.globalDict['GAME']['state'] = 'menu_quit'
        utils.clearGlobalDict()
        scene.replace('lan_server')
    elif what == 24:
        # test des personnages :
        bge.logic.globalDict['GAME']['state'] = 'menu_quit'
        utils.saveSettings()
        #gameName = bge.logic.globalDict['MENU']['gameName']
        #bge.logic.globalDict['GAME']['levels'] = {}
        #bge.logic.globalDict['GAME']['createdOjects'] = []
        scene.replace('characters')
    elif what == 25:
        # lancer Blender :
        try:
            import subprocess
            blender = sys.argv[0].replace('blenderplayer', 'blender')
            process = subprocess.Popen([blender])
        except:
            print('EXECUTION FAILED')


# **************************************************
#       FONCTION PRINCIPALE
# **************************************************
def main():
    # ---------------------------------------------------------
    # RÉCUPÉRATION DE L'ÉTAT DU JEU
    # ---------------------------------------------------------
    # si on lance le fichier directement
    if not('GAME' in bge.logic.globalDict):
        bge.logic.globalDict['GAME'] = {}
    state = bge.logic.globalDict['GAME'].get('state', '')
    if not(state in ('menu_first', 'menu_loop', 'menu_quit')):
        bge.logic.globalDict['GAME']['state'] = 'menu_first'

    # ---------------------------------------------------------
    # PREMIER LANCEMENT
    # ---------------------------------------------------------
    if bge.logic.globalDict['GAME']['state'] == 'menu_first':
        # valeurs initiales (lues depuis le fichier de config si existe) :
        utils.clearGlobalDict()
        utils.loadSettings()
        lang = bge.logic.globalDict['MENU']['lang']
        global controller, scene
        controller = bge.logic.getCurrentController()
        scene = bge.logic.getCurrentScene()
        # affichage des textes :
        scene.post_draw += [postDrawTexts]
        bge.logic.globalDict['TEXTS'] = {}
        utils.setText(
            'progName', text=utils.PROGNAME, 
            color=utils.colors['blue'], width=3, position=(0.05, 0.12))
        utils.setText('comment', color=utils.colors['blue'], width=2, position=(0.1, 0.83))
        # création des menus :
        bge.logic.globalDict['TEMP']['menuItems'] = {}
        bge.logic.globalDict['TEMP']['menuOver'] = ''
        for menuItemName in menuItems:
            what = int(menuItemName[5:])
            if what == 1:
                menuText = utils.tr('games', bge.logic.globalDict['MENU']['gameName'], lang)
            elif what == 2:
                menuText = utils.tr('menu', bge.logic.globalDict['MENU']['character'], lang)
            elif what == 3:
                menuText = utils.tr('menu', bge.logic.globalDict['MENU']['mode'], lang)
            else:
                menuText = utils.tr('menu', menuItems[menuItemName][0], lang)
            # si le menu est cochable, on initialise son état :
            if menuItems[menuItemName][2]:
                menuKey = menuItems[menuItemName][5]
                if not(menuKey in bge.logic.globalDict['MENU']):
                    bge.logic.globalDict['MENU'][menuKey] = False
                if bge.logic.globalDict['MENU'][menuKey]:
                    menuText = '[X]    {0}'.format(menuText)
                else:
                    menuText = '[ ]    {0}'.format(menuText)
            utils.createMenuItem(
                menuItemName, 
                name=menuItems[menuItemName][0], 
                text=menuText,
                width=menuItems[menuItemName][3], 
                position=menuItems[menuItemName][1], 
                what=what)

        # pour les tests :
        bge.logic.globalDict['TEMP']['TEST_KEY'] = False
        # on affiche la souris :
        bge.logic.mouse.position = bge.logic.globalDict['GAME']['cursorPosition']
        bge.render.showMouse(True)
        # initialisation terminée :
        bge.logic.globalDict['GAME']['state'] = 'menu_loop'

    # ---------------------------------------------------------
    # BOUCLE DES ÉVÉNEMENTS
    # ---------------------------------------------------------
    if bge.logic.globalDict['GAME']['state'] != 'menu_loop':
        return
    # récupération des coordonnées de la souris :
    cursorPosition = input_controller.getCursorPosition()
    # affichage des menus et récupération de l'action active :
    what = -1
    for menuItemName in bge.logic.globalDict['TEMP']['menuItems']:
        menuItem = bge.logic.globalDict['TEMP']['menuItems'][menuItemName]
        menuPosition = menuItem['position']
        over = input_controller.mouseOver(cursorPosition, menuPosition)
        if over:
            what = menuItem['what']
            lang = bge.logic.globalDict['MENU']['lang']
            utils.setText('comment', text=utils.tr('menu', menuItems[menuItemName][4], lang))
            if menuItemName != bge.logic.globalDict['TEMP']['menuOver']:
                bge.logic.globalDict['TEMP']['menuOver'] = menuItemName
                menuItem['width'] = menuItem['initialWidth'] * 1.5
                utils.setText(menuItemName, width=menuItem['width'])
                utils.playSound('over')
        else:
            menuItem['width'] = menuItem['initialWidth']
            utils.setText(menuItemName, width=menuItem['width'])
            if menuItemName == bge.logic.globalDict['TEMP']['menuOver']:
                bge.logic.globalDict['TEMP']['menuOver'] = ''
    inputState = input_controller.inputManagement('menu')
    # pour faire un screencast :
    if inputState['SCREENSHOT']:
        utils.makeScreenshot()
    # gestion des menus et autres actions :
    if (what > 0) and (inputState['ENTER']):
        utils.playSound('clic')
        doAction(what)
    elif inputState['ENTER']:
        doAction(91)
    elif inputState['HELP']:
        doAction(21)
    elif inputState['QUIT']:
        doAction(92)
    elif inputState['NEXT']:
        menuItem = 'item_{0}'.format(what)
        menuItem = utils.nextInList(menuItemsList, menuItem)
        menuPosition = bge.logic.globalDict['TEMP']['menuItems'][menuItem]['position']
        bge.logic.mouse.position = (menuPosition[0] + 0.02, menuPosition[1])


